@@ -705,7 +705,8 @@ get_definition(vahControl * ctl, const char *xmlStr)
     }
 
     ctl->def = virDomainDefParseString(xmlStr,
-                                       ctl->caps, ctl->xmlopt,
+                                       ctl->caps, ctl->xmlopt, NULL,
+                                       VIR_DOMAIN_DEF_PARSE_SKIP_SECLABEL |
                                        VIR_DOMAIN_DEF_PARSE_SKIP_VALIDATE);
 
     if (ctl->def == NULL) {
@@ -741,6 +742,7 @@ vah_add_path(virBufferPtr buf, const char *path, const char *perms, bool recursi
     bool readonly = true;
     bool explicit_deny_rule = true;
     char *sub = NULL;
+    char *perms_new = NULL;
 
     if (path == NULL)
         return rc;
@@ -765,12 +767,15 @@ vah_add_path(virBufferPtr buf, const char *path, const char *perms, bool recursi
         return rc;
     }
 
-    if (strchr(perms, 'w') != NULL) {
+    if (VIR_STRDUP_QUIET(perms_new, perms) < 0)
+        goto cleanup;
+
+    if (strchr(perms_new, 'w') != NULL) {
         readonly = false;
         explicit_deny_rule = false;
     }
 
-    if ((sub = strchr(perms, 'R')) != NULL) {
+    if ((sub = strchr(perms_new, 'R')) != NULL) {
         /* Don't write the invalid R permission, replace it with 'r' */
         sub[0] = 'r';
         explicit_deny_rule = false;
@@ -788,7 +793,8 @@ vah_add_path(virBufferPtr buf, const char *path, const char *perms, bool recursi
     if (tmp[strlen(tmp) - 1] == '/')
         tmp[strlen(tmp) - 1] = '\0';
 
-    virBufferAsprintf(buf, "  \"%s%s\" %s,\n", tmp, recursive ? "/**" : "", perms);
+    virBufferAsprintf(buf, "  \"%s%s\" %s,\n", tmp, recursive ? "/**" : "",
+                      perms_new);
     if (explicit_deny_rule) {
         virBufferAddLit(buf, "  # don't audit writes to readonly files\n");
         virBufferAsprintf(buf, "  deny \"%s%s\" w,\n", tmp, recursive ? "/**" : "");
@@ -799,6 +805,7 @@ vah_add_path(virBufferPtr buf, const char *path, const char *perms, bool recursi
     }
 
  cleanup:
+    VIR_FREE(perms_new);
     VIR_FREE(tmp);
 
     return rc;
@@ -941,45 +948,45 @@ get_files(vahControl * ctl)
 
     for (i = 0; i < ctl->def->nserials; i++)
         if (ctl->def->serials[i] &&
-            (ctl->def->serials[i]->source.type == VIR_DOMAIN_CHR_TYPE_PTY ||
-             ctl->def->serials[i]->source.type == VIR_DOMAIN_CHR_TYPE_DEV ||
-             ctl->def->serials[i]->source.type == VIR_DOMAIN_CHR_TYPE_FILE ||
-             ctl->def->serials[i]->source.type == VIR_DOMAIN_CHR_TYPE_UNIX ||
-             ctl->def->serials[i]->source.type == VIR_DOMAIN_CHR_TYPE_PIPE) &&
-            ctl->def->serials[i]->source.data.file.path &&
-            ctl->def->serials[i]->source.data.file.path[0] != '\0')
+            (ctl->def->serials[i]->source->type == VIR_DOMAIN_CHR_TYPE_PTY ||
+             ctl->def->serials[i]->source->type == VIR_DOMAIN_CHR_TYPE_DEV ||
+             ctl->def->serials[i]->source->type == VIR_DOMAIN_CHR_TYPE_FILE ||
+             ctl->def->serials[i]->source->type == VIR_DOMAIN_CHR_TYPE_UNIX ||
+             ctl->def->serials[i]->source->type == VIR_DOMAIN_CHR_TYPE_PIPE) &&
+            ctl->def->serials[i]->source->data.file.path &&
+            ctl->def->serials[i]->source->data.file.path[0] != '\0')
             if (vah_add_file_chardev(&buf,
-                                     ctl->def->serials[i]->source.data.file.path,
+                                     ctl->def->serials[i]->source->data.file.path,
                                      "rw",
-                                     ctl->def->serials[i]->source.type) != 0)
+                                     ctl->def->serials[i]->source->type) != 0)
                 goto cleanup;
 
     for (i = 0; i < ctl->def->nconsoles; i++)
         if (ctl->def->consoles[i] &&
-            (ctl->def->consoles[i]->source.type == VIR_DOMAIN_CHR_TYPE_PTY ||
-             ctl->def->consoles[i]->source.type == VIR_DOMAIN_CHR_TYPE_DEV ||
-             ctl->def->consoles[i]->source.type == VIR_DOMAIN_CHR_TYPE_FILE ||
-             ctl->def->consoles[i]->source.type == VIR_DOMAIN_CHR_TYPE_UNIX ||
-             ctl->def->consoles[i]->source.type == VIR_DOMAIN_CHR_TYPE_PIPE) &&
-            ctl->def->consoles[i]->source.data.file.path &&
-            ctl->def->consoles[i]->source.data.file.path[0] != '\0')
+            (ctl->def->consoles[i]->source->type == VIR_DOMAIN_CHR_TYPE_PTY ||
+             ctl->def->consoles[i]->source->type == VIR_DOMAIN_CHR_TYPE_DEV ||
+             ctl->def->consoles[i]->source->type == VIR_DOMAIN_CHR_TYPE_FILE ||
+             ctl->def->consoles[i]->source->type == VIR_DOMAIN_CHR_TYPE_UNIX ||
+             ctl->def->consoles[i]->source->type == VIR_DOMAIN_CHR_TYPE_PIPE) &&
+            ctl->def->consoles[i]->source->data.file.path &&
+            ctl->def->consoles[i]->source->data.file.path[0] != '\0')
             if (vah_add_file(&buf,
-                             ctl->def->consoles[i]->source.data.file.path, "rw") != 0)
+                             ctl->def->consoles[i]->source->data.file.path, "rw") != 0)
                 goto cleanup;
 
     for (i = 0; i < ctl->def->nparallels; i++)
         if (ctl->def->parallels[i] &&
-            (ctl->def->parallels[i]->source.type == VIR_DOMAIN_CHR_TYPE_PTY ||
-             ctl->def->parallels[i]->source.type == VIR_DOMAIN_CHR_TYPE_DEV ||
-             ctl->def->parallels[i]->source.type == VIR_DOMAIN_CHR_TYPE_FILE ||
-             ctl->def->parallels[i]->source.type == VIR_DOMAIN_CHR_TYPE_UNIX ||
-             ctl->def->parallels[i]->source.type == VIR_DOMAIN_CHR_TYPE_PIPE) &&
-            ctl->def->parallels[i]->source.data.file.path &&
-            ctl->def->parallels[i]->source.data.file.path[0] != '\0')
+            (ctl->def->parallels[i]->source->type == VIR_DOMAIN_CHR_TYPE_PTY ||
+             ctl->def->parallels[i]->source->type == VIR_DOMAIN_CHR_TYPE_DEV ||
+             ctl->def->parallels[i]->source->type == VIR_DOMAIN_CHR_TYPE_FILE ||
+             ctl->def->parallels[i]->source->type == VIR_DOMAIN_CHR_TYPE_UNIX ||
+             ctl->def->parallels[i]->source->type == VIR_DOMAIN_CHR_TYPE_PIPE) &&
+            ctl->def->parallels[i]->source->data.file.path &&
+            ctl->def->parallels[i]->source->data.file.path[0] != '\0')
             if (vah_add_file_chardev(&buf,
-                                     ctl->def->parallels[i]->source.data.file.path,
+                                     ctl->def->parallels[i]->source->data.file.path,
                                      "rw",
-                                     ctl->def->parallels[i]->source.type) != 0)
+                                     ctl->def->parallels[i]->source->type) != 0)
                 goto cleanup;
 
     virBufferAsprintf(&buf, "  # for qemu guest agent channel\n");
@@ -988,17 +995,17 @@ get_files(vahControl * ctl)
 
     for (i = 0; i < ctl->def->nchannels; i++)
         if (ctl->def->channels[i] &&
-            (ctl->def->channels[i]->source.type == VIR_DOMAIN_CHR_TYPE_PTY ||
-             ctl->def->channels[i]->source.type == VIR_DOMAIN_CHR_TYPE_DEV ||
-             ctl->def->channels[i]->source.type == VIR_DOMAIN_CHR_TYPE_FILE ||
-             ctl->def->channels[i]->source.type == VIR_DOMAIN_CHR_TYPE_UNIX ||
-             ctl->def->channels[i]->source.type == VIR_DOMAIN_CHR_TYPE_PIPE) &&
-            ctl->def->channels[i]->source.data.file.path &&
-            ctl->def->channels[i]->source.data.file.path[0] != '\0')
+            (ctl->def->channels[i]->source->type == VIR_DOMAIN_CHR_TYPE_PTY ||
+             ctl->def->channels[i]->source->type == VIR_DOMAIN_CHR_TYPE_DEV ||
+             ctl->def->channels[i]->source->type == VIR_DOMAIN_CHR_TYPE_FILE ||
+             ctl->def->channels[i]->source->type == VIR_DOMAIN_CHR_TYPE_UNIX ||
+             ctl->def->channels[i]->source->type == VIR_DOMAIN_CHR_TYPE_PIPE) &&
+            ctl->def->channels[i]->source->data.file.path &&
+            ctl->def->channels[i]->source->data.file.path[0] != '\0')
             if (vah_add_file_chardev(&buf,
-                                     ctl->def->channels[i]->source.data.file.path,
+                                     ctl->def->channels[i]->source->data.file.path,
                                      "rw",
-                                     ctl->def->channels[i]->source.type) != 0)
+                                     ctl->def->channels[i]->source->type) != 0)
                 goto cleanup;
 
     if (ctl->def->os.kernel)
