@@ -88,6 +88,7 @@ libxlDriverConfigDispose(void *obj)
     VIR_FREE(cfg->saveDir);
     VIR_FREE(cfg->autoDumpDir);
     VIR_FREE(cfg->lockManagerName);
+    VIR_FREE(cfg->channelDir);
     virFirmwareFreeList(cfg->firmwares, cfg->nfirmwares);
 }
 
@@ -207,8 +208,8 @@ libxlMakeDomCreateInfo(libxl_ctx *ctx,
 static int
 libxlMakeChrdevStr(virDomainChrDefPtr def, char **buf)
 {
-    virDomainChrSourceDef srcdef = def->source;
-    const char *type = virDomainChrTypeToString(srcdef.type);
+    virDomainChrSourceDefPtr srcdef = def->source;
+    const char *type = virDomainChrTypeToString(srcdef->type);
 
     if (!type) {
         virReportError(VIR_ERR_CONFIG_UNSUPPORTED,
@@ -216,7 +217,7 @@ libxlMakeChrdevStr(virDomainChrDefPtr def, char **buf)
         return -1;
     }
 
-    switch (srcdef.type) {
+    switch (srcdef->type) {
     case VIR_DOMAIN_CHR_TYPE_NULL:
     case VIR_DOMAIN_CHR_TYPE_STDIO:
     case VIR_DOMAIN_CHR_TYPE_VC:
@@ -227,19 +228,19 @@ libxlMakeChrdevStr(virDomainChrDefPtr def, char **buf)
 
     case VIR_DOMAIN_CHR_TYPE_FILE:
     case VIR_DOMAIN_CHR_TYPE_PIPE:
-        if (virAsprintf(buf, "%s:%s", type, srcdef.data.file.path) < 0)
+        if (virAsprintf(buf, "%s:%s", type, srcdef->data.file.path) < 0)
             return -1;
         break;
 
     case VIR_DOMAIN_CHR_TYPE_DEV:
-        if (VIR_STRDUP(*buf, srcdef.data.file.path) < 0)
+        if (VIR_STRDUP(*buf, srcdef->data.file.path) < 0)
             return -1;
         break;
 
     case VIR_DOMAIN_CHR_TYPE_UDP: {
-        const char *connectHost = srcdef.data.udp.connectHost;
-        const char *bindHost = srcdef.data.udp.bindHost;
-        const char *bindService  = srcdef.data.udp.bindService;
+        const char *connectHost = srcdef->data.udp.connectHost;
+        const char *bindHost = srcdef->data.udp.bindHost;
+        const char *bindService  = srcdef->data.udp.bindService;
 
         if (connectHost == NULL)
             connectHost = "";
@@ -250,7 +251,7 @@ libxlMakeChrdevStr(virDomainChrDefPtr def, char **buf)
 
         if (virAsprintf(buf, "udp:%s:%s@%s:%s",
                         connectHost,
-                        srcdef.data.udp.connectService,
+                        srcdef->data.udp.connectService,
                         bindHost,
                         bindService) < 0)
             return -1;
@@ -260,24 +261,24 @@ libxlMakeChrdevStr(virDomainChrDefPtr def, char **buf)
     case VIR_DOMAIN_CHR_TYPE_TCP: {
         const char *prefix;
 
-        if (srcdef.data.tcp.protocol == VIR_DOMAIN_CHR_TCP_PROTOCOL_TELNET)
+        if (srcdef->data.tcp.protocol == VIR_DOMAIN_CHR_TCP_PROTOCOL_TELNET)
             prefix = "telnet";
         else
             prefix = "tcp";
 
         if (virAsprintf(buf, "%s:%s:%s%s",
                         prefix,
-                        srcdef.data.tcp.host,
-                        srcdef.data.tcp.service,
-                        srcdef.data.tcp.listen ? ",server,nowait" : "") < 0)
+                        srcdef->data.tcp.host,
+                        srcdef->data.tcp.service,
+                        srcdef->data.tcp.listen ? ",server,nowait" : "") < 0)
             return -1;
         break;
     }
 
     case VIR_DOMAIN_CHR_TYPE_UNIX:
         if (virAsprintf(buf, "unix:%s%s",
-                        srcdef.data.nix.path,
-                        srcdef.data.nix.listen ? ",server,nowait" : "") < 0)
+                        srcdef->data.nix.path,
+                        srcdef->data.nix.listen ? ",server,nowait" : "") < 0)
             return -1;
         break;
 
@@ -298,6 +299,7 @@ libxlMakeDomBuildInfo(virDomainDefPtr def,
     libxl_domain_build_info *b_info = &d_config->b_info;
     int hvm = def->os.type == VIR_DOMAIN_OSTYPE_HVM;
     size_t i;
+    size_t nusbdevice = 0;
 
     libxl_domain_build_info_init(b_info);
 
@@ -431,14 +433,31 @@ libxlMakeDomBuildInfo(virDomainDefPtr def,
         }
 
         if (def->nserials) {
-            if (def->nserials > 1) {
+            if (def->nserials == 1) {
+                if (libxlMakeChrdevStr(def->serials[0], &b_info->u.hvm.serial) <
+                    0)
+                    return -1;
+            } else {
+#ifdef LIBXL_HAVE_BUILDINFO_SERIAL_LIST
+                if (VIR_ALLOC_N(b_info->u.hvm.serial_list, def->nserials + 1) <
+                    0)
+                    return -1;
+                for (i = 0; i < def->nserials; i++) {
+                    if (libxlMakeChrdevStr(def->serials[i],
+                                           &b_info->u.hvm.serial_list[i]) < 0)
+                    {
+                        libxl_string_list_dispose(&b_info->u.hvm.serial_list);
+                        return -1;
+                    }
+                }
+                b_info->u.hvm.serial_list[i] = NULL;
+#else
                 virReportError(VIR_ERR_CONFIG_UNSUPPORTED,
                                "%s",
                                _("Only one serial device is supported by libxl"));
                 return -1;
+#endif
             }
-            if (libxlMakeChrdevStr(def->serials[0], &b_info->u.hvm.serial) < 0)
-                return -1;
         }
 
         if (def->nparallels) {
@@ -452,56 +471,59 @@ libxlMakeDomBuildInfo(virDomainDefPtr def,
         libxl_defbool_set(&b_info->u.hvm.vnc.enable, 0);
         libxl_defbool_set(&b_info->u.hvm.sdl.enable, 0);
 
-        if (def->ninputs) {
+        for (i = 0; i < def->ninputs; i++) {
+            char **usbdevice;
+
+            if (def->inputs[i]->bus != VIR_DOMAIN_INPUT_BUS_USB)
+                continue;
+
 #ifdef LIBXL_HAVE_BUILDINFO_USBDEVICE_LIST
-            if (VIR_ALLOC_N(b_info->u.hvm.usbdevice_list, def->ninputs+1) < 0)
+            if (VIR_EXPAND_N(b_info->u.hvm.usbdevice_list, nusbdevice, 1) < 0)
                 return -1;
 #else
-            if (def->ninputs > 1) {
+            if (i > 1) {
                 virReportError(VIR_ERR_CONFIG_UNSUPPORTED, "%s",
                         _("libxenlight supports only one input device"));
                 return -1;
             }
 #endif
-            for (i = 0; i < def->ninputs; i++) {
-                char **usbdevice;
-
-                if (def->inputs[i]->bus != VIR_DOMAIN_INPUT_BUS_USB)
-                    continue;
 
 #ifdef LIBXL_HAVE_BUILDINFO_USBDEVICE_LIST
-                usbdevice = &b_info->u.hvm.usbdevice_list[i];
+            usbdevice = &b_info->u.hvm.usbdevice_list[i];
 #else
-                usbdevice = &b_info->u.hvm.usbdevice;
+            usbdevice = &b_info->u.hvm.usbdevice;
 #endif
-                switch (def->inputs[i]->type) {
-                    case VIR_DOMAIN_INPUT_TYPE_MOUSE:
-                        VIR_FREE(*usbdevice);
-                        if (VIR_STRDUP(*usbdevice, "mouse") < 0)
-                            return -1;
-                        break;
-                    case VIR_DOMAIN_INPUT_TYPE_TABLET:
-                        VIR_FREE(*usbdevice);
-                        if (VIR_STRDUP(*usbdevice, "tablet") < 0)
-                            return -1;
-                        break;
-                    default:
-                        virReportError(VIR_ERR_CONFIG_UNSUPPORTED, "%s",
-                                _("Unknown input device type"));
+            switch (def->inputs[i]->type) {
+                case VIR_DOMAIN_INPUT_TYPE_MOUSE:
+                    VIR_FREE(*usbdevice);
+                    if (VIR_STRDUP(*usbdevice, "mouse") < 0)
                         return -1;
-                }
+                    break;
+                case VIR_DOMAIN_INPUT_TYPE_TABLET:
+                    VIR_FREE(*usbdevice);
+                    if (VIR_STRDUP(*usbdevice, "tablet") < 0)
+                        return -1;
+                    break;
+                default:
+                    virReportError(VIR_ERR_CONFIG_UNSUPPORTED, "%s",
+                            _("Unknown input device type"));
+                    return -1;
             }
         }
 
-        /*
-         * The following comment and calculation were taken directly from
-         * libxenlight's internal function libxl_get_required_shadow_memory():
-         *
-         * 256 pages (1MB) per vcpu, plus 1 page per MiB of RAM for the P2M map,
-         * plus 1 page per MiB of RAM to shadow the resident processes.
-         */
-        b_info->shadow_memkb = 4 * (256 * libxl_bitmap_count_set(&b_info->avail_vcpus) +
-                                    2 * (b_info->max_memkb / 1024));
+#ifdef LIBXL_HAVE_BUILDINFO_USBDEVICE_LIST
+        /* NULL-terminate usbdevice_list */
+        if (nusbdevice > 0 &&
+            VIR_EXPAND_N(b_info->u.hvm.usbdevice_list, nusbdevice, 1) < 0) {
+            VIR_DISPOSE_N(b_info->u.hvm.usbdevice_list, nusbdevice);
+            return -1;
+        }
+#endif
+
+        /* Allow libxl to calculate shadow memory requirements */
+        b_info->shadow_memkb =
+            libxl_get_required_shadow_memory(b_info->max_memkb,
+                                             b_info->max_vcpus);
     } else {
         /*
          * For compatibility with the legacy xen toolstack, default to pygrub
@@ -1327,6 +1349,8 @@ libxlDriverConfigNew(void)
         goto error;
     if (VIR_STRDUP(cfg->autoDumpDir, LIBXL_DUMP_DIR) < 0)
         goto error;
+    if (VIR_STRDUP(cfg->channelDir, LIBXL_CHANNEL_DIR) < 0)
+        goto error;
 
     if (virAsprintf(&log_file, "%s/libxl-driver.log", cfg->logDir) < 0)
         goto error;
@@ -1478,28 +1502,223 @@ int libxlDriverConfigLoadFile(libxlDriverConfigPtr cfg,
 
 }
 
+#ifdef LIBXL_HAVE_DEVICE_CHANNEL
+static int
+libxlPrepareChannel(virDomainChrDefPtr channel,
+                    const char *channelDir,
+                    const char *domainName)
+{
+    if (channel->targetType == VIR_DOMAIN_CHR_CHANNEL_TARGET_TYPE_XEN &&
+        channel->source->type == VIR_DOMAIN_CHR_TYPE_UNIX &&
+        !channel->source->data.nix.path) {
+        if (virAsprintf(&channel->source->data.nix.path,
+                        "%s/%s-%s", channelDir, domainName,
+                        channel->target.name ? channel->target.name
+                        : "unknown.sock") < 0)
+            return -1;
+
+        channel->source->data.nix.listen = true;
+    }
+
+    return 0;
+}
+
+static int
+libxlMakeChannel(virDomainChrDefPtr l_channel,
+                 libxl_device_channel *x_channel)
+{
+    libxl_device_channel_init(x_channel);
+
+    if (l_channel->targetType != VIR_DOMAIN_CHR_CHANNEL_TARGET_TYPE_XEN) {
+        virReportError(VIR_ERR_CONFIG_UNSUPPORTED, "%s",
+                       _("channel target type not supported"));
+        return -1;
+    }
+
+    switch (l_channel->source->type) {
+    case VIR_DOMAIN_CHR_TYPE_PTY:
+        x_channel->connection = LIBXL_CHANNEL_CONNECTION_PTY;
+        break;
+    case VIR_DOMAIN_CHR_TYPE_UNIX:
+        x_channel->connection = LIBXL_CHANNEL_CONNECTION_SOCKET;
+        if (VIR_STRDUP(x_channel->u.socket.path,
+                       l_channel->source->data.nix.path) < 0)
+            return -1;
+        break;
+    default:
+        virReportError(VIR_ERR_CONFIG_UNSUPPORTED, "%s",
+                       _("channel source type not supported"));
+        break;
+    }
+
+    if (!l_channel->target.name) {
+        virReportError(VIR_ERR_OPERATION_INVALID, "%s",
+                       _("channel target name missing"));
+        return -1;
+    }
+
+    if (VIR_STRDUP(x_channel->name, l_channel->target.name) < 0)
+        return -1;
+
+    return 0;
+}
+
+static int
+libxlMakeChannelList(const char *channelDir,
+                     virDomainDefPtr def,
+                     libxl_domain_config *d_config)
+{
+    virDomainChrDefPtr *l_channels = def->channels;
+    size_t nchannels = def->nchannels;
+    libxl_device_channel *x_channels;
+    size_t i, nvchannels = 0;
+
+    if (VIR_ALLOC_N(x_channels, nchannels) < 0)
+        return -1;
+
+    for (i = 0; i < nchannels; i++) {
+        if (l_channels[i]->deviceType != VIR_DOMAIN_CHR_DEVICE_TYPE_CHANNEL)
+            continue;
+
+        if (libxlPrepareChannel(l_channels[i], channelDir, def->name) < 0)
+            goto error;
+
+        if (libxlMakeChannel(l_channels[i], &x_channels[nvchannels]) < 0)
+            goto error;
+
+        nvchannels++;
+    }
+
+    VIR_SHRINK_N(x_channels, nchannels, nchannels - nvchannels);
+    d_config->channels = x_channels;
+    d_config->num_channels = nvchannels;
+
+    return 0;
+
+ error:
+    for (i = 0; i < nchannels; i++)
+        libxl_device_channel_dispose(&x_channels[i]);
+    VIR_FREE(x_channels);
+    return -1;
+}
+#endif
+
 #ifdef LIBXL_HAVE_PVUSB
+int
+libxlMakeUSBController(virDomainControllerDefPtr controller,
+                       libxl_device_usbctrl *usbctrl)
+{
+    usbctrl->devid = controller->idx;
+
+    if (controller->type != VIR_DOMAIN_CONTROLLER_TYPE_USB)
+        return -1;
+
+    if (controller->model == -1) {
+        usbctrl->version = 2;
+        usbctrl->type = LIBXL_USBCTRL_TYPE_QUSB;
+    } else {
+        switch (controller->model) {
+        case VIR_DOMAIN_CONTROLLER_MODEL_USB_QUSB1:
+            usbctrl->version = 1;
+            usbctrl->type = LIBXL_USBCTRL_TYPE_QUSB;
+            break;
+
+        case VIR_DOMAIN_CONTROLLER_MODEL_USB_QUSB2:
+            usbctrl->version = 2;
+            usbctrl->type = LIBXL_USBCTRL_TYPE_QUSB;
+            break;
+
+        default:
+            virReportError(VIR_ERR_CONFIG_UNSUPPORTED, "%s",
+                           _("unsupported usb model"));
+            return -1;
+        }
+    }
+
+    if (controller->opts.usbopts.ports == -1)
+        usbctrl->ports = 8;
+    else
+        usbctrl->ports = controller->opts.usbopts.ports;
+
+    return 0;
+}
+
+static int
+libxlMakeUSBControllerList(virDomainDefPtr def, libxl_domain_config *d_config)
+{
+    virDomainControllerDefPtr *l_controllers = def->controllers;
+    size_t ncontrollers = def->ncontrollers;
+    size_t nusbctrls = 0;
+    libxl_device_usbctrl *x_usbctrls;
+    size_t i;
+
+    if (ncontrollers == 0)
+        return 0;
+
+    if (VIR_ALLOC_N(x_usbctrls, ncontrollers) < 0)
+        return -1;
+
+    for (i = 0; i < ncontrollers; i++) {
+        if (l_controllers[i]->type != VIR_DOMAIN_CONTROLLER_TYPE_USB)
+            continue;
+
+        libxl_device_usbctrl_init(&x_usbctrls[nusbctrls]);
+
+        if (libxlMakeUSBController(l_controllers[i],
+                                   &x_usbctrls[nusbctrls]) < 0)
+            goto error;
+
+        nusbctrls++;
+    }
+
+    VIR_SHRINK_N(x_usbctrls, ncontrollers, ncontrollers - nusbctrls);
+    d_config->usbctrls = x_usbctrls;
+    d_config->num_usbctrls = nusbctrls;
+
+    return 0;
+
+ error:
+    for (i = 0; i < nusbctrls; i++)
+        libxl_device_usbctrl_dispose(&x_usbctrls[i]);
+
+    VIR_FREE(x_usbctrls);
+    return -1;
+}
+
 int
 libxlMakeUSB(virDomainHostdevDefPtr hostdev, libxl_device_usbdev *usbdev)
 {
     virDomainHostdevSubsysUSBPtr usbsrc = &hostdev->source.subsys.u.usb;
+    virUSBDevicePtr usb = NULL;
+    int ret = -1;
 
     if (hostdev->mode != VIR_DOMAIN_HOSTDEV_MODE_SUBSYS)
-        return -1;
+        return ret;
     if (hostdev->source.subsys.type != VIR_DOMAIN_HOSTDEV_SUBSYS_TYPE_USB)
-        return -1;
+        return ret;
 
-    if (usbsrc->bus <= 0 || usbsrc->device <= 0) {
-        virReportError(VIR_ERR_CONFIG_UNSUPPORTED, "%s",
-                       _("libxenlight supports only USB device "
-                         "specified by busnum:devnum"));
-        return -1;
+    if (usbsrc->bus > 0 && usbsrc->device > 0) {
+        usbdev->u.hostdev.hostbus = usbsrc->bus;
+        usbdev->u.hostdev.hostaddr = usbsrc->device;
+    } else {
+        if (virHostdevFindUSBDevice(hostdev, true, &usb) < 0) {
+            virReportError(VIR_ERR_OPERATION_FAILED,
+                           _("failed to find USB device busnum:devnum "
+                             "for %x:%x"),
+                           usbsrc->vendor, usbsrc->product);
+            goto cleanup;
+        }
+
+        usbdev->u.hostdev.hostbus = virUSBDeviceGetBus(usb);
+        usbdev->u.hostdev.hostaddr = virUSBDeviceGetDevno(usb);
     }
 
-    usbdev->u.hostdev.hostbus = usbsrc->bus;
-    usbdev->u.hostdev.hostaddr = usbsrc->device;
+    ret = 0;
 
-    return 0;
+ cleanup:
+    virUSBDeviceFree(usb);
+
+    return ret;
 }
 
 static int
@@ -1718,6 +1937,7 @@ libxlDriverNodeGetInfo(libxlDriverPrivatePtr driver, virNodeInfoPtr info)
 int
 libxlBuildDomainConfig(virPortAllocatorPtr graphicsports,
                        virDomainDefPtr def,
+                       const char *channelDir LIBXL_ATTR_UNUSED,
                        libxl_ctx *ctx,
                        libxl_domain_config *d_config)
 {
@@ -1745,10 +1965,18 @@ libxlBuildDomainConfig(virPortAllocatorPtr graphicsports,
         return -1;
 
 #ifdef LIBXL_HAVE_PVUSB
+    if (libxlMakeUSBControllerList(def, d_config) < 0)
+        return -1;
+
     if (libxlMakeUSBList(def, d_config) < 0)
         return -1;
 #endif
 
+#ifdef LIBXL_HAVE_DEVICE_CHANNEL
+    if (libxlMakeChannelList(channelDir, def, d_config) < 0)
+        return -1;
+#endif
+
     /*
      * Now that any potential VFBs are defined, update the build info with
      * the data of the primary display. Some day libxl might implicitely do
