@@ -707,7 +707,7 @@ get_definition(vahControl * ctl, const char *xmlStr)
     }
 
     ctl->def = virDomainDefParseString(xmlStr,
-                                       ctl->caps, ctl->xmlopt,
+                                       ctl->caps, ctl->xmlopt, NULL,
                                        VIR_DOMAIN_DEF_PARSE_SKIP_SECLABEL |
                                        VIR_DOMAIN_DEF_PARSE_SKIP_VALIDATE);
 
@@ -737,14 +737,14 @@ get_definition(vahControl * ctl, const char *xmlStr)
   * read with no explicit deny rule.
   */
 static int
-vah_add_path(virBufferPtr buf, const char *path, const char *inperms, bool recursive)
+vah_add_path(virBufferPtr buf, const char *path, const char *perms, bool recursive)
 {
     char *tmp = NULL;
-    char *perms = NULL;
     int rc = -1;
     bool readonly = true;
     bool explicit_deny_rule = true;
     char *sub = NULL;
+    char *perms_new = NULL;
 
     if (path == NULL)
         return rc;
@@ -759,27 +759,25 @@ vah_add_path(virBufferPtr buf, const char *path, const char *inperms, bool recur
         return 0;
     }
 
-    if (VIR_STRDUP_QUIET(perms, inperms) < 0)
-        return rc;
-
     if (virFileExists(path)) {
         if ((tmp = realpath(path, NULL)) == NULL) {
             vah_error(NULL, 0, path);
             vah_error(NULL, 0, _("could not find realpath for disk"));
-            VIR_FREE(perms);
             return rc;
         }
     } else if (VIR_STRDUP_QUIET(tmp, path) < 0) {
-        VIR_FREE(perms);
         return rc;
     }
 
-    if (strchr(perms, 'w') != NULL) {
+    if (VIR_STRDUP_QUIET(perms_new, perms) < 0)
+        goto cleanup;
+
+    if (strchr(perms_new, 'w') != NULL) {
         readonly = false;
         explicit_deny_rule = false;
     }
 
-    if ((sub = strchr(perms, 'R')) != NULL) {
+    if ((sub = strchr(perms_new, 'R')) != NULL) {
         /* Don't write the invalid R permission, replace it with 'r' */
         sub[0] = 'r';
         explicit_deny_rule = false;
@@ -797,7 +795,8 @@ vah_add_path(virBufferPtr buf, const char *path, const char *inperms, bool recur
     if (tmp[strlen(tmp) - 1] == '/')
         tmp[strlen(tmp) - 1] = '\0';
 
-    virBufferAsprintf(buf, "  \"%s%s\" %s,\n", tmp, recursive ? "/**" : "", perms);
+    virBufferAsprintf(buf, "  \"%s%s\" %s,\n", tmp, recursive ? "/**" : "",
+                      perms_new);
     if (explicit_deny_rule) {
         virBufferAddLit(buf, "  # don't audit writes to readonly files\n");
         virBufferAsprintf(buf, "  deny \"%s%s\" w,\n", tmp, recursive ? "/**" : "");
@@ -808,8 +807,8 @@ vah_add_path(virBufferPtr buf, const char *path, const char *inperms, bool recur
     }
 
  cleanup:
+    VIR_FREE(perms_new);
     VIR_FREE(tmp);
-    VIR_FREE(perms);
 
     return rc;
 }
@@ -951,45 +950,45 @@ get_files(vahControl * ctl)
 
     for (i = 0; i < ctl->def->nserials; i++)
         if (ctl->def->serials[i] &&
-            (ctl->def->serials[i]->source.type == VIR_DOMAIN_CHR_TYPE_PTY ||
-             ctl->def->serials[i]->source.type == VIR_DOMAIN_CHR_TYPE_DEV ||
-             ctl->def->serials[i]->source.type == VIR_DOMAIN_CHR_TYPE_FILE ||
-             ctl->def->serials[i]->source.type == VIR_DOMAIN_CHR_TYPE_UNIX ||
-             ctl->def->serials[i]->source.type == VIR_DOMAIN_CHR_TYPE_PIPE) &&
-            ctl->def->serials[i]->source.data.file.path &&
-            ctl->def->serials[i]->source.data.file.path[0] != '\0')
+            (ctl->def->serials[i]->source->type == VIR_DOMAIN_CHR_TYPE_PTY ||
+             ctl->def->serials[i]->source->type == VIR_DOMAIN_CHR_TYPE_DEV ||
+             ctl->def->serials[i]->source->type == VIR_DOMAIN_CHR_TYPE_FILE ||
+             ctl->def->serials[i]->source->type == VIR_DOMAIN_CHR_TYPE_UNIX ||
+             ctl->def->serials[i]->source->type == VIR_DOMAIN_CHR_TYPE_PIPE) &&
+            ctl->def->serials[i]->source->data.file.path &&
+            ctl->def->serials[i]->source->data.file.path[0] != '\0')
             if (vah_add_file_chardev(&buf,
-                                     ctl->def->serials[i]->source.data.file.path,
+                                     ctl->def->serials[i]->source->data.file.path,
                                      "rw",
-                                     ctl->def->serials[i]->source.type) != 0)
+                                     ctl->def->serials[i]->source->type) != 0)
                 goto cleanup;
 
     for (i = 0; i < ctl->def->nconsoles; i++)
         if (ctl->def->consoles[i] &&
-            (ctl->def->consoles[i]->source.type == VIR_DOMAIN_CHR_TYPE_PTY ||
-             ctl->def->consoles[i]->source.type == VIR_DOMAIN_CHR_TYPE_DEV ||
-             ctl->def->consoles[i]->source.type == VIR_DOMAIN_CHR_TYPE_FILE ||
-             ctl->def->consoles[i]->source.type == VIR_DOMAIN_CHR_TYPE_UNIX ||
-             ctl->def->consoles[i]->source.type == VIR_DOMAIN_CHR_TYPE_PIPE) &&
-            ctl->def->consoles[i]->source.data.file.path &&
-            ctl->def->consoles[i]->source.data.file.path[0] != '\0')
+            (ctl->def->consoles[i]->source->type == VIR_DOMAIN_CHR_TYPE_PTY ||
+             ctl->def->consoles[i]->source->type == VIR_DOMAIN_CHR_TYPE_DEV ||
+             ctl->def->consoles[i]->source->type == VIR_DOMAIN_CHR_TYPE_FILE ||
+             ctl->def->consoles[i]->source->type == VIR_DOMAIN_CHR_TYPE_UNIX ||
+             ctl->def->consoles[i]->source->type == VIR_DOMAIN_CHR_TYPE_PIPE) &&
+            ctl->def->consoles[i]->source->data.file.path &&
+            ctl->def->consoles[i]->source->data.file.path[0] != '\0')
             if (vah_add_file(&buf,
-                             ctl->def->consoles[i]->source.data.file.path, "rw") != 0)
+                             ctl->def->consoles[i]->source->data.file.path, "rw") != 0)
                 goto cleanup;
 
     for (i = 0; i < ctl->def->nparallels; i++)
         if (ctl->def->parallels[i] &&
-            (ctl->def->parallels[i]->source.type == VIR_DOMAIN_CHR_TYPE_PTY ||
-             ctl->def->parallels[i]->source.type == VIR_DOMAIN_CHR_TYPE_DEV ||
-             ctl->def->parallels[i]->source.type == VIR_DOMAIN_CHR_TYPE_FILE ||
-             ctl->def->parallels[i]->source.type == VIR_DOMAIN_CHR_TYPE_UNIX ||
-             ctl->def->parallels[i]->source.type == VIR_DOMAIN_CHR_TYPE_PIPE) &&
-            ctl->def->parallels[i]->source.data.file.path &&
-            ctl->def->parallels[i]->source.data.file.path[0] != '\0')
+            (ctl->def->parallels[i]->source->type == VIR_DOMAIN_CHR_TYPE_PTY ||
+             ctl->def->parallels[i]->source->type == VIR_DOMAIN_CHR_TYPE_DEV ||
+             ctl->def->parallels[i]->source->type == VIR_DOMAIN_CHR_TYPE_FILE ||
+             ctl->def->parallels[i]->source->type == VIR_DOMAIN_CHR_TYPE_UNIX ||
+             ctl->def->parallels[i]->source->type == VIR_DOMAIN_CHR_TYPE_PIPE) &&
+            ctl->def->parallels[i]->source->data.file.path &&
+            ctl->def->parallels[i]->source->data.file.path[0] != '\0')
             if (vah_add_file_chardev(&buf,
-                                     ctl->def->parallels[i]->source.data.file.path,
+                                     ctl->def->parallels[i]->source->data.file.path,
                                      "rw",
-                                     ctl->def->parallels[i]->source.type) != 0)
+                                     ctl->def->parallels[i]->source->type) != 0)
                 goto cleanup;
 
     virBufferAsprintf(&buf, "  # for qemu guest agent channel\n");
@@ -998,17 +997,17 @@ get_files(vahControl * ctl)
 
     for (i = 0; i < ctl->def->nchannels; i++)
         if (ctl->def->channels[i] &&
-            (ctl->def->channels[i]->source.type == VIR_DOMAIN_CHR_TYPE_PTY ||
-             ctl->def->channels[i]->source.type == VIR_DOMAIN_CHR_TYPE_DEV ||
-             ctl->def->channels[i]->source.type == VIR_DOMAIN_CHR_TYPE_FILE ||
-             ctl->def->channels[i]->source.type == VIR_DOMAIN_CHR_TYPE_UNIX ||
-             ctl->def->channels[i]->source.type == VIR_DOMAIN_CHR_TYPE_PIPE) &&
-            ctl->def->channels[i]->source.data.file.path &&
-            ctl->def->channels[i]->source.data.file.path[0] != '\0')
+            (ctl->def->channels[i]->source->type == VIR_DOMAIN_CHR_TYPE_PTY ||
+             ctl->def->channels[i]->source->type == VIR_DOMAIN_CHR_TYPE_DEV ||
+             ctl->def->channels[i]->source->type == VIR_DOMAIN_CHR_TYPE_FILE ||
+             ctl->def->channels[i]->source->type == VIR_DOMAIN_CHR_TYPE_UNIX ||
+             ctl->def->channels[i]->source->type == VIR_DOMAIN_CHR_TYPE_PIPE) &&
+            ctl->def->channels[i]->source->data.file.path &&
+            ctl->def->channels[i]->source->data.file.path[0] != '\0')
             if (vah_add_file_chardev(&buf,
-                                     ctl->def->channels[i]->source.data.file.path,
+                                     ctl->def->channels[i]->source->data.file.path,
                                      "rw",
-                                     ctl->def->channels[i]->source.type) != 0)
+                                     ctl->def->channels[i]->source->type) != 0)
                 goto cleanup;
 
     if (ctl->def->os.kernel)
